/************************************************************************
 ***   GROOVESIZER ECHO v.2 - development template with the Mozzi library
 ***   http://sensorium.github.io/Mozzi/
 ***   for the GROOVESIZER 8-Bit Musical Multiboard
 ***   http://groovesizer.com 
 ************************************************************************
 * Copyright (C) 2013 MoShang (Jean Marais) moshang@groovesizer.com
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/
 * 
 ************************************************************************/

/*  Example playing a sinewave at a set frequency,
 *  using Mozzi sonification library.
 *
 *  Demonstrates the use of Oscil to play a wavetable.
 *
 *  IMPORTANT: this sketch requires Mozzi/mozzi_config.h to be
 *  be changed from STANDARD mode to HIFI.
 *  In Mozz/mozzi_config.h, change
 *  #define AUDIO_MODE STANDARD
 *  //#define AUDIO_MODE HIFI
 *  to
 *  //#define AUDIO_MODE STANDARD
 *  #define AUDIO_MODE HIFI
 *
 *
 *  Circuit: Audio output on digital pin 9 and 10 (on a Uno or similar),
 *  Check the Mozzi core module documentation for others and more detail
 *
 *                   3.9k 
 *   pin 9  ---WWWW-----|-----output
 *                     1M           |
 *   pin 10 ---WWWW---- |
 *                                     |
 *                           4.7n  ==
 *                                     |
 *                                 ground
 *
 *  Resistors are ±0.5%  Measure and choose the most precise 
 *  from a batch of whatever you can get.
 *
 *
 *  Mozzi help/discussion/announcements:
 *  https://groups.google.com/forum/#!forum/mozzi-users
 *
 *  Tim Barrass 2012-13.
 *  This example code is in the public domain.
 */

#define AUDIO_MODE HIFI
#include <MozziGuts.h>
#include <Oscil.h> // oscillator template
#include <tables/sin2048_int8.h> // sine table for oscillator
#include <mozzi_midi.h>
//#include <Wire.h>               // I2C library (required by the EEPROM library below)
//#include <EEPROM24LC256_512.h>  // https://github.com/mikeneiderhauser/I2C-EEPROM-Arduino

//EEPROM256_512 mem_1;            // define the eeprom chip
//byte rwBuffer[64];              // our EEPROM read/write buffer

// use: Oscil <table_size, update_rate> oscilName (wavetable)
Oscil <SIN2048_NUM_CELLS, AUDIO_RATE> aSin(SIN2048_DATA);

Oscil <2048, CONTROL_RATE> kVib(SIN2048_DATA);

float centre_freq = 440.0;
float depth = 0.35;
float vibFreq = 0.5;

// 4051 for the potentiometers
// array of pins used to select 1 of 8 inputs on multiplexer
const int select[] = {
  17,16,15}; // pins connected to the 4051 input select lines
const int analogPin = 0;      // the analog pin connected to multiplexer output

// required by the button checker
byte BUTTONlatchPin = 7;
byte BUTTONclockPin = 8;
byte BUTTONdataPin = 3; // for Echo firmware
#define DEBOUNCE 5 // the debounce time in milliseconds
byte pressed[32], justpressed[32], justreleased[32], buttons[32];
//define the byte variables that the button states are read into
byte BUTTONvar1; 
byte BUTTONvar2; 
byte BUTTONvar3; 
byte BUTTONvar4;
byte BUTTONvar5;

// required for UpdateLEDs
int LEDlatchPin = 2;
int LEDclockPin = 6; //for mozzi
int LEDdataPin = 4;

byte LEDrow1 = B00000000;
byte LEDrow2 = B00000000;
byte LEDrow3 = B00000000;
byte LEDrow4 = B00000000;
byte LEDrow5 = B00000000;

byte buttonLED[5];

#define CONTROL_RATE 256

void setup(){
  startMozzi(CONTROL_RATE); // uses the default control rate of 64, defined in mozzi_config.h
  aSin.setFreq(440); // set the frequency
  kVib.setFreq(vibFreq);

  // required by the button checker
  pinMode(BUTTONlatchPin, OUTPUT);
  pinMode(BUTTONclockPin, OUTPUT); 
  pinMode(BUTTONdataPin, INPUT);

  // required for UpdateLEDs
  pinMode(LEDlatchPin, OUTPUT);
  pinMode(LEDclockPin, OUTPUT); 
  pinMode(LEDdataPin, OUTPUT);

  for(int bit = 0; bit < 3; bit++)
    pinMode(select[bit], OUTPUT);  // set the three EEPROM select pins to output

  //begin I2C Bus
 // Wire.begin();

  //begin EEPROM with I2C Address 
  //mem_1.begin(0,0);//addr 0 (DEC) type 0 (defined as 24LC256)
  
  //scrollText(" ECHO V02  ");
}


void updateControl(){
  static int pot1;
  static int pot2;
  static int pot3;
  static int freq;
  static unsigned long mozziTime;
  static unsigned long checkTime;
  
  static byte controlDivider = 0;
  
  controlDivider++;
  controlDivider = controlDivider%6;
  
  if (controlDivider == 0)
  {
  pot1 = constrain(getValue(0), 30, 1000);
  freq = map(pot1, 30, 1000, 70, 5000);
  //freq = (pot1 < 40) ? 0 : pot1; 
  }
  
  if (controlDivider == 2)
  {
  pot2 = constrain(getValue(1), 30, 1000);
  pot2 = (pot2 < 35) ? 0 : pot2;
  depth = float(pot2)/100.0;
  }
  if (controlDivider == 4)
  {
  pot3 = getValue(2);
  vibFreq = float(pot3)/100.0; 
  }
  
  kVib.setFreq(vibFreq);
  float vibrato = depth * kVib.next();
  aSin.setFreq(freq + vibrato); // set the frequency
  checkTime = mozziTime;

  check_switches();

  for (byte i = 0; i < 40; i++)
  {
    if (pressed[i])
    {
      aSin.setFreq(1000+(i*50)); // set the frequency
    }
  }
  for (byte i = 0; i < 5; i++)
    buttonLED[i] = 0;

  byte LEDvalue = map(freq, 70, 5000, 0, 40); // we also want to know how pot 6 relates to our 40 LEDs
  for (byte i = 0; i < 40; i++)
  {
    if (i <= LEDvalue)
      bitSet(buttonLED[i/8], i%8); // turn LEDs on that are lower or equal to the pot value
  }

  // update the actual bytes we're going to shift out on this loop
  LEDrow1 =  buttonLED[0];
  LEDrow2 =  buttonLED[1];
  LEDrow3 =  buttonLED[2];
  LEDrow4 =  buttonLED[3];
  LEDrow5 =  buttonLED[4];

  updateLeds();
}


int updateAudio(){
  // this would make more sense with a higher resolution signal
  // but still benefits from using HIFI to avoid the 16kHz pwm noise
  return aSin.next()<<6; // 8 bits scaled up to 14 bits
}


void loop(){


  audioHook(); // required here
}








