
/**********************************************************************************************
 ***
 *** Button checker with shift-in for 4 CD4021Bs parallel to serial shift registers (32 buttons)
 *** Adapted from Adafruit http://www.adafruit.com/blog/2009/10/20/example-code-for-multi-button-checker-with-debouncing/
 *** ShiftIn based on  http://www.arduino.cc/en/Tutorial/ShiftIn
 ***
 ***********************************************************************************************/
//***TOP*OF*THE*SKETCH*********************************************************************
// remember to add the following to the start of your sketch before (before setup)
/*
byte BUTTONlatchPin = 7;
 byte BUTTONclockPin = 8;
 byte BUTTONdataPin = 3; // for Echo firmware
 #define DEBOUNCE 5 // the debounce time in milliseconds
 byte pressed[32], justpressed[32], justreleased[32], buttons[32];
 //define the byte variables that the button states are read into
 byte BUTTONvar1; 
 byte BUTTONvar2; 
 byte BUTTONvar3; 
 byte BUTTONvar4;
 */
//***SETUP*************************************************************************************
// and add the following pin modes to setup()
/*
pinMode(BUTTONlatchPin, OUTPUT);
 pinMode(BUTTONclockPin, OUTPUT); 
 pinMode(BUTTONdataPin, INPUT);
 */
//***SHIFTIN***********************************************************************************
// definition of the shiftIn function
// just needs the location of the data pin and the clock pin
// it returns a byte with each bit in the byte corresponding
// to a pin on the shift register. leftBit 7 = Pin 7 / Bit 0= Pin 0

byte shiftIn(int myDataPin, int myClockPin) { 
  int i;
  int temp = 0;
  int pinState;
  byte myDataIn = 0;

  pinMode(myClockPin, OUTPUT);
  pinMode(myDataPin, INPUT);

  //we will be holding the clock pin high 8 times (0,..,7) at the
  //end of each time through the for loop

  //at the begining of each loop when we set the clock low, it will
  //be doing the necessary low to high drop to cause the shift
  //register's DataPin to change state based on the value
  //of the next bit in its serial information flow.
  //The register transmits the information about the pins from pin 7 to pin 0
  //so that is why our function counts down
  for (i=0; i<=7; i++)
  {
    digitalWrite(myClockPin, 0);
    //delayMicroseconds(2);
    temp = digitalRead(myDataPin);
    if (temp) {
      pinState = 1;
      //set the bit to 0 no matter what
      myDataIn = myDataIn | (1 << i);
    }
    else {
      //turn it off -- only necessary for debuging
      //print statement since myDataIn starts as 0
      pinState = 0;
    }

    //Debuging print statements
    //Serial.print(pinState);
    //Serial.print("     ");
    //Serial.println (dataIn, BIN);

    digitalWrite(myClockPin, 1);

  }
  //debuging print statements whitespace
  //Serial.println();
  //Serial.println(myDataIn, BIN);
  return myDataIn;
}

//*********************************************************************************************

void check_switches()
{
  static byte previousstate[40];
  static byte currentstate[40];
  static long lasttime;
  byte index;


/*
  if (millis() < lasttime) {
    // we wrapped around, lets just try again
    lasttime = millis();
  }

  if ((lasttime + DEBOUNCE) > millis()) {
    // not enough time has passed to debounce
    return; 
  }
  // ok we have waited DEBOUNCE milliseconds, lets reset the timer
  lasttime = millis();
*/

  //BUTTONS
  //Collect button data from the 4901s
  //Pulse the latch pin:
  //set it to 1 to collect parallel data
  digitalWrite(BUTTONlatchPin,1);
  //set it to 1 to collect parallel data, wait
  delayMicroseconds(20);
  //set it to 0 to transmit data serially  
  digitalWrite(BUTTONlatchPin,0);

  //while the shift register is in serial mode
  //collect each shift register into a byte
  //the register attached to the chip comes in first 
  BUTTONvar1 = shiftIn(BUTTONdataPin, BUTTONclockPin);
  BUTTONvar2 = shiftIn(BUTTONdataPin, BUTTONclockPin);
  BUTTONvar3 = shiftIn(BUTTONdataPin, BUTTONclockPin);
  BUTTONvar4 = shiftIn(BUTTONdataPin, BUTTONclockPin);
  BUTTONvar5 = shiftIn(BUTTONdataPin, BUTTONclockPin);

  //write the button states to the currentstate array
  for (int i = 0; i<=7; i++)
  {
    buttons[i] = BUTTONvar1 & (1 << i);
  }
  for (int i = 8; i<=15; i++)
  {
    buttons[i] = BUTTONvar2 & (1 << (i%8));
  }
  for (int i = 16; i<=23; i++)
  {
    buttons[i] = BUTTONvar3 & (1 << (i%8));
  }
  for (int i = 24; i<=31; i++)
  {
    buttons[i] = BUTTONvar4 & (1 << (i%8));
  }
  for (int i = 32; i<=39; i++)
  {
    buttons[i] = BUTTONvar5 & (1 << (i%8));
  }

  for (index = 0; index < 40; index++) {
    justpressed[index] = 0;       // when we start, we clear out the "just" indicators
    justreleased[index] = 0;

    //currentstate[index] = buttons[index];   // read the button state from the array

    currentstate[index] = (buttons[index] == 0) ? HIGH : LOW;

    if (currentstate[index] == previousstate[index]) {
      if ((pressed[index] == LOW) && (currentstate[index] == LOW)) {
        // just pressed
        justpressed[index] = 1;
      }
      else if ((pressed[index] == HIGH) && (currentstate[index] == HIGH)) {
        // just released
        justreleased[index] = 1;
      }
      pressed[index] = !currentstate[index];  // remember, digital HIGH means NOT pressed
    }
    //Serial.println(pressed[index], DEC);
    previousstate[index] = currentstate[index];   // keep a running tally of the buttons
  }
}

// POTS
// reading from 4051
// this function returns the analog value for the given channel
int getValue( int channel)
{
  // set the selector pins HIGH and LOW to match the binary value of channel
  for(int bit = 0; bit < 3; bit++)
  {
    int pin = select[bit]; // the pin wired to the multiplexer select bit
    int isBitSet =  bitRead(channel, bit); // true if given bit set in channel
    digitalWrite(pin, isBitSet);
  }
  //return map((constrain(analogRead(analogPin), 40, 1000)), 40, 1000, 0, 1023); // we're only using readings between 40 and 1000, as the extreme ends are unstable
  int reading = analogRead(analogPin);
  return reading;
}


/**********************************************************************************************
 ***
 *** ShiftOut to address 42 LEDs via 4 74HC595s serial to parallel shifting registers 
 *** Based on http://arduino.cc/en/Tutorial/ShiftOut
 ***
 ***********************************************************************************************/
//***TOP*OF*THE*SKETCH**************************************************************************
// remember to add the following to the start of your sketch before (before setup)
/*
int LEDlatchPin = 2;
 int LEDclockPin = 6; for mozzi
 int LEDdataPin = 4;
 */
//***SETUP**************************************************************************************
// and add the following pin modes to setup()
/*
pinMode(LEDlatchPin, OUTPUT);
 pinMode(LEDclockPin, OUTPUT); 
 pinMode(LEDdataPin, OUTPUT);
 */

void shiftOut(int myLEDdataPin, int myLEDclockPin, byte myDataOut) {
  // This shifts 8 bits out MSB first, 
  // on the rising edge of the clock,
  // clock idles low

  //internal function setup
  int i=0;
  int pinState;
  pinMode(myLEDclockPin, OUTPUT);
  pinMode(myLEDdataPin, OUTPUT);

  //clear everything out just in case to
  //prepare shift register for bit shifting
  digitalWrite(myLEDdataPin, 0);
  digitalWrite(myLEDclockPin, 0);

  //for each bit in the byte myDataOut
  //NOTICE THAT WE ARE COUNTING DOWN in our for loop
  //This means that %00000001 or "1" will go through such
  //that it will be pin Q0 that lights. 
  for (i=7; i>=0; i--)  {
    digitalWrite(myLEDclockPin, 0);

    //if the value passed to myDataOut and a bitmask result 
    // true then... so if we are at i=6 and our value is
    // %11010100 it would the code compares it to %01000000 
    // and proceeds to set pinState to 1.
    if ( myDataOut & (1<<i) ) {
      pinState= 1;
    }
    else {	
      pinState= 0;
    }

    //Sets the pin to HIGH or LOW depending on pinState
    digitalWrite(myLEDdataPin, pinState);
    //register shifts bits on upstroke of clock pin  
    digitalWrite(myLEDclockPin, 1);
    //zero the data pin after shift to prevent bleed through
    digitalWrite(myLEDdataPin, 0);
  }

  //stop shifting
  digitalWrite(myLEDclockPin, 0);
}

// this function shifts out the the 4 bytes corresponding to the led rows
void updateLeds(void)
{
  static boolean lastSentTop = false; // we want to alternate sending the top 2 and bottom 3 rows to prevent an edge case where 4 rows of LEDs lit at the same time sourcing too much current
  //ground LEDlatchPin and hold low for as long as you are transmitting
  digitalWrite(LEDlatchPin, 0);
  if (!lastSentTop) // send the top to rows
  {
    shiftOut(LEDdataPin, LEDclockPin, B00000000);
    shiftOut(LEDdataPin, LEDclockPin, B00000000); 
    shiftOut(LEDdataPin, LEDclockPin, B00000000);
    shiftOut(LEDdataPin, LEDclockPin, LEDrow2); 
    shiftOut(LEDdataPin, LEDclockPin, LEDrow1);
    lastSentTop = true;
  }
  else // ie. lastSentTop is true, then send the bottom 3 rows
  {
    shiftOut(LEDdataPin, LEDclockPin, LEDrow5);
    shiftOut(LEDdataPin, LEDclockPin, LEDrow4); 
    shiftOut(LEDdataPin, LEDclockPin, LEDrow3);
    shiftOut(LEDdataPin, LEDclockPin, B00000000); 
    shiftOut(LEDdataPin, LEDclockPin, B00000000);
    lastSentTop = false;
  }
  //return the latch pin high to signal chip that it 
  //no longer needs to listen for information
  digitalWrite(LEDlatchPin, 1);
}
/*
void scrollText(char myString[])
{
  byte textLength = strlen(myString);
  // read character definitions from EEPROM
  for (byte j = 0; j < 1; j++) // how many times from 0 - 9 & A - 7
  {
    for (byte m = 0; m < textLength; m++) // how many characters
    {
      int page = 508 ;
      byte c = myString[m];
      if (c < 0)       // SPACE
      {
        c = 0;
        page = 508;
      }
      else if (c < 58)  // 0 - 10
      {
        c = (c - 47)*5;
        page = 508;
      }
      else if (c < 65)  // ?
      {
        c = 55;
      }
      else if (c < 77) // A - L
      {
        c = (c - 65)*5;
        page = 509;
      }
      else if (c < 89) // M - X
      {
        c = (c - 77)*5;
        page = 510;
      }
      else
      {
        c = (c - 89)*5;
        page = 511;
      }
      mem_1.readPage(page, rwBuffer); // read the page that the character is on
      byte charRow1 =  rwBuffer[c];
      byte charRow2 =  rwBuffer[c+1];
      byte charRow3 =  rwBuffer[c+2];
      byte charRow4 =  rwBuffer[c+3];
      byte charRow5 =  rwBuffer[c+4];

      for (byte i = 0; i < 7; i++) // how many pixels wide?
        // it's only 7 because we don't need three blank pixels before the character, 2 are enough
      {
        //shift each row one pixel (bit) to the right to make space for the new collumn 
        LEDrow1 = LEDrow1 >> 1;
        LEDrow2 = LEDrow2 >> 1;
        LEDrow3 = LEDrow3 >> 1;
        LEDrow4 = LEDrow4 >> 1;
        LEDrow5 = LEDrow5 >> 1;

        if (bitRead(charRow1, 6-i)) // shorthand for "if ((bitRead(charRow1, i) == 1)"
          bitSet(LEDrow1, 7);
        if (bitRead(charRow2, 6-i)) // 
          bitSet(LEDrow2, 7);
        if (bitRead(charRow3, 6-i))
          bitSet(LEDrow3, 7);
        if (bitRead(charRow4, 6-i))
          bitSet(LEDrow4, 7);
        if (bitRead(charRow5, 6-i))
          bitSet(LEDrow5, 7);

        for (int k = 0; k < 10; k++) // how many times through the update cycle before we move to the next vertical row of pixels
        {
          updateLeds();
          delay(5);
        }
      }
    }
  }
}
*/

